@@ -48,11 +48,11 @@ from pytensor.graph.basic import (
 )
 from pytensor.graph.rewriting.basic import GraphRewriter, NodeRewriter
 from pytensor.graph.traversal import ancestors, walk
-from pytensor.tensor.variable import TensorVariable
 
 from pymc.logprob.abstract import (
     MeasurableOp,
     _icdf_helper,
+    _logccdf_helper,
     _logcdf_helper,
     _logprob,
     _logprob_helper,
@@ -81,7 +81,7 @@ def _find_unallowed_rvs_in_graph(graph):
     }
 
 
-def _warn_rvs_in_inferred_graph(graph: TensorVariable | Sequence[TensorVariable]):
+def _warn_rvs_in_inferred_graph(graph: Variable | Sequence[Variable]):
     """Issue warning if any RVs are found in graph.
 
     RVs are usually an (implicit) conditional input of the derived probability expression,
@@ -102,14 +102,14 @@ def _warn_rvs_in_inferred_graph(graph: TensorVariable | Sequence[TensorVariable]
         )
 
 
-def logp(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> TensorVariable:
+def logp(rv: Variable, value: Variable | TensorLike, warn_rvs=True, **kwargs) -> Variable:
     """Create a graph for the log-probability of a random variable.
 
     Parameters
     ----------
-    rv : TensorVariable
-    value : tensor_like
-        Should be the same type (shape and dtype) as the rv.
+    rv : Variable
+    value : Variable or tensor_like
+        Should be the same type as the rv.
     warn_rvs : bool, default True
         Warn if RVs were found in the logp graph.
         This can happen when a variable has other other random variables as inputs.
@@ -118,7 +118,7 @@ def logp(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Tens
 
     Returns
     -------
-    logp : TensorVariable
+    logp : Variable
 
     Raises
     ------
@@ -188,7 +188,8 @@ def logp(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Tens
             pm.CustomDist("x", mu, sigma, logp=normal_logp)
 
     """
-    value = pt.as_tensor_variable(value, dtype=rv.dtype)
+    if not isinstance(value, Variable):
+        value = pt.as_tensor_variable(value, dtype=rv.dtype)
     try:
         return _logprob_helper(rv, value, **kwargs)
     except NotImplementedError:
@@ -202,14 +203,14 @@ def logp(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Tens
         return expr
 
 
-def logcdf(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> TensorVariable:
+def logcdf(rv: Variable, value: Variable | TensorLike, warn_rvs=True, **kwargs) -> Variable:
     """Create a graph for the log-CDF of a random variable.
 
     Parameters
     ----------
-    rv : TensorVariable
+    rv : Variable
     value : tensor_like
-        Should be the same type (shape and dtype) as the rv.
+        Should be the same type as the rv.
     warn_rvs : bool, default True
         Warn if RVs were found in the logcdf graph.
         This can happen when a variable has other random variables as inputs.
@@ -217,7 +218,7 @@ def logcdf(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Te
 
     Returns
     -------
-    logp : TensorVariable
+    logp : Variable
 
     Raises
     ------
@@ -287,7 +288,8 @@ def logcdf(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Te
             pm.CustomDist("x", mu, sigma, logcdf=normal_logcdf)
 
     """
-    value = pt.as_tensor_variable(value, dtype=rv.dtype)
+    if not isinstance(value, Variable):
+        value = pt.as_tensor_variable(value, dtype=rv.dtype)
     try:
         return _logcdf_helper(rv, value, **kwargs)
     except NotImplementedError:
@@ -302,14 +304,79 @@ def logcdf(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Te
         return expr
 
 
-def icdf(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> TensorVariable:
+def logccdf(rv: Variable, value: Variable | TensorLike, warn_rvs=True, **kwargs) -> Variable:
+    """Create a graph for the log complementary CDF (log survival function) of a random variable.
+
+    The log complementary CDF is defined as log(1 - CDF(x)), also known as the
+    log survival function. For distributions with a numerically stable implementation,
+    this is more accurate than computing log(1 - exp(logcdf)).
+
+    Parameters
+    ----------
+    rv : Variable
+    value : tensor_like
+        Should be the same type (shape and dtype) as the rv.
+    warn_rvs : bool, default True
+        Warn if RVs were found in the logccdf graph.
+        This can happen when a variable has other random variables as inputs.
+        In that case, those random variables should be replaced by their respective values.
+
+    Returns
+    -------
+    logccdf : Variable
+
+    Raises
+    ------
+    RuntimeError
+        If the logccdf cannot be derived.
+
+    Examples
+    --------
+    Create a compiled function that evaluates the logccdf of a variable
+
+    .. code-block:: python
+
+        import pymc as pm
+        import pytensor.tensor as pt
+
+        mu = pt.scalar("mu")
+        rv = pm.Normal.dist(mu, 1.0)
+
+        value = pt.scalar("value")
+        rv_logccdf = pm.logccdf(rv, value)
+
+        # Use .eval() for debugging
+        print(rv_logccdf.eval({value: 0.9, mu: 0.0}))  # -1.5272506
+
+        # Compile a function for repeated evaluations
+        rv_logccdf_fn = pm.compile_pymc([value, mu], rv_logccdf)
+        print(rv_logccdf_fn(value=0.9, mu=0.0))  # -1.5272506
+
+    """
+    if not isinstance(value, Variable):
+        value = pt.as_tensor_variable(value, dtype=rv.dtype)
+    try:
+        return _logccdf_helper(rv, value, **kwargs)
+    except NotImplementedError:
+        # Try to rewrite rv
+        fgraph = construct_ir_fgraph({rv: value})
+        [ir_valued_rv] = fgraph.outputs
+        [ir_rv, ir_value] = ir_valued_rv.owner.inputs
+        expr = _logccdf_helper(ir_rv, ir_value, **kwargs)
+        [expr] = cleanup_ir([expr])
+        if warn_rvs:
+            _warn_rvs_in_inferred_graph([expr])
+        return expr
+
+
+def icdf(rv: Variable, value: Variable | TensorLike, warn_rvs=True, **kwargs) -> Variable:
     """Create a graph for the inverse CDF of a random variable.
 
     Parameters
     ----------
-    rv : TensorVariable
+    rv : Variable
     value : tensor_like
-        Should be the same type (shape and dtype) as the rv.
+        Should be the same type as the rv, except dtype can differ.
     warn_rvs : bool, default True
         Warn if RVs were found in the icdf graph.
         This can happen when a variable has other random variables as inputs.
@@ -317,7 +384,7 @@ def icdf(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Tens
 
     Returns
     -------
-    icdf : TensorVariable
+    icdf : Variable
 
     Raises
     ------
@@ -369,7 +436,8 @@ def icdf(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Tens
         print(exp_rv_icdf_fn(value=0.9, mu=0.0))  # 3.60222448
 
     """
-    value = pt.as_tensor_variable(value, dtype="floatX")
+    if not isinstance(value, Variable):
+        value = pt.as_tensor_variable(value, dtype="floatX")
     try:
         return _icdf_helper(rv, value, **kwargs)
     except NotImplementedError:
@@ -385,12 +453,12 @@ def icdf(rv: TensorVariable, value: TensorLike, warn_rvs=True, **kwargs) -> Tens
 
 
 def conditional_logp(
-    rv_values: dict[TensorVariable, TensorVariable],
+    rv_values: dict[Variable, Variable],
     warn_rvs=True,
     ir_rewriter: GraphRewriter | None = None,
     extra_rewrites: GraphRewriter | NodeRewriter | None = None,
     **kwargs,
-) -> dict[TensorVariable, TensorVariable]:
+) -> dict[Variable, Variable]:
     r"""Create a map between variables and conditional logps such that the sum is their joint logp.
 
     The `rv_values` dictionary specifies a joint probability graph defined by
@@ -548,13 +616,13 @@ def conditional_logp(
 
 
 def transformed_conditional_logp(
-    rvs: Sequence[TensorVariable],
+    rvs: Sequence[Variable],
     *,
-    rvs_to_values: dict[TensorVariable, TensorVariable],
-    rvs_to_transforms: dict[TensorVariable, Transform],
+    rvs_to_values: dict[Variable, Variable],
+    rvs_to_transforms: dict[Variable, Transform],
     jacobian: bool = True,
     **kwargs,
-) -> list[TensorVariable]:
+) -> list[Variable]:
     """Thin wrapper around conditional_logprob, which creates a value transform rewrite.
 
     This helper will only return the subset of logprob terms corresponding to `rvs`.
